import 'dart:math';

import 'package:flutter/material.dart';

import 'package:momomotus/config/default_game_settings.dart';
import 'package:momomotus/models/settings/settings_game.dart';
import 'package:momomotus/models/settings/settings_global.dart';
import 'package:momomotus/utils/tools.dart';

class ParameterPainter extends CustomPainter {
  const ParameterPainter({
    required this.code,
    required this.value,
    required this.isSelected,
    required this.gameSettings,
    required this.globalSettings,
  });

  final String code;
  final String value;
  final bool isSelected;
  final GameSettings gameSettings;
  final GlobalSettings globalSettings;

  @override
  void paint(Canvas canvas, Size size) {
    // force square
    final double canvasSize = min(size.width, size.height);

    const Color borderColorEnabled = Colors.blue;
    const Color borderColorDisabled = Colors.white;

    // "enabled/disabled" border
    final paint = Paint();
    paint.style = PaintingStyle.stroke;
    paint.color = isSelected ? borderColorEnabled : borderColorDisabled;
    paint.strokeJoin = StrokeJoin.round;
    paint.strokeWidth = 10;
    canvas.drawRect(
        Rect.fromPoints(const Offset(0, 0), Offset(canvasSize, canvasSize)), paint);

    // content
    switch (code) {
      case DefaultGameSettings.parameterCodeLang:
        paintLangParameterItem(value, canvas, canvasSize);
        break;
      case DefaultGameSettings.parameterCodeLength:
        paintLengthParameterItem(value, canvas, canvasSize);
        break;
      case DefaultGameSettings.parameterCodeLevel:
        paintLevelParameterItem(value, canvas, canvasSize);
        break;
      default:
        printlog('Unknown parameter: $code/$value');
        paintUnknownParameterItem(value, canvas, canvasSize);
    }
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return false;
  }

  // "unknown" parameter -> simple block with text
  void paintUnknownParameterItem(
    final String value,
    final Canvas canvas,
    final double size,
  ) {
    final paint = Paint();
    paint.strokeJoin = StrokeJoin.round;
    paint.strokeWidth = 3;

    paint.color = Colors.grey;
    paint.style = PaintingStyle.fill;
    canvas.drawRect(Rect.fromPoints(const Offset(0, 0), Offset(size, size)), paint);

    final textSpan = TextSpan(
      text: '?\n$value',
      style: const TextStyle(
        color: Colors.black,
        fontSize: 18,
        fontWeight: FontWeight.bold,
      ),
    );
    final textPainter = TextPainter(
      text: textSpan,
      textDirection: TextDirection.ltr,
      textAlign: TextAlign.center,
    );
    textPainter.layout();
    textPainter.paint(
      canvas,
      Offset(
        (size - textPainter.width) * 0.5,
        (size - textPainter.height) * 0.5,
      ),
    );
  }

  void paintLangParameterItem(
    final String value,
    final Canvas canvas,
    final double size,
  ) {
    const itemCountEmoji = '💬\n';

    Color backgroundColor = Colors.grey;
    String text = '';

    switch (value) {
      case DefaultGameSettings.lengthValue4:
        backgroundColor = Colors.grey;
        text = '⭐';
        break;
      case DefaultGameSettings.lengthValue5:
        backgroundColor = Colors.green;
        text = itemCountEmoji + DefaultGameSettings.lengthValue5.toString();
        break;
      case DefaultGameSettings.lengthValue6:
        backgroundColor = Colors.orange;
        text = itemCountEmoji + DefaultGameSettings.lengthValue6.toString();
        break;
      case DefaultGameSettings.lengthValue7:
        backgroundColor = Colors.red;
        text = itemCountEmoji + DefaultGameSettings.lengthValue7.toString();
        break;
      default:
        printlog('Wrong value for lang parameter value: $value');
        return;
    }

    final paint = Paint();
    paint.strokeJoin = StrokeJoin.round;
    paint.strokeWidth = 3 / 100 * size;

    // Colored background
    paint.color = backgroundColor;
    paint.style = PaintingStyle.fill;
    canvas.drawRect(Rect.fromPoints(const Offset(0, 0), Offset(size, size)), paint);

    // centered text value
    final textSpan = TextSpan(
      text: text,
      style: TextStyle(
        color: Colors.black,
        fontSize: size / 2.6,
        fontWeight: FontWeight.bold,
      ),
    );
    final textPainter = TextPainter(
      text: textSpan,
      textDirection: TextDirection.ltr,
      textAlign: TextAlign.center,
    );
    textPainter.layout();
    textPainter.paint(
      canvas,
      Offset(
        (size - textPainter.width) * 0.5,
        (size - textPainter.height) * 0.5,
      ),
    );
  }

  void paintLengthParameterItem(
    final String value,
    final Canvas canvas,
    final double size,
  ) {
    const itemCountEmoji = '💬\n';

    Color backgroundColor = Colors.grey;
    final String text = itemCountEmoji + value.toString();

    switch (value) {
      case DefaultGameSettings.lengthValue4:
        backgroundColor = Colors.green;
        break;
      case DefaultGameSettings.lengthValue5:
        backgroundColor = Colors.yellow;
        break;
      case DefaultGameSettings.lengthValue6:
        backgroundColor = Colors.orange;
        break;
      case DefaultGameSettings.lengthValue7:
        backgroundColor = Colors.red;
        break;
      case DefaultGameSettings.lengthValue8:
        backgroundColor = Colors.purple;
        break;
      default:
        printlog('Wrong value for length parameter value: $value');
        return;
    }

    final paint = Paint();
    paint.strokeJoin = StrokeJoin.round;
    paint.strokeWidth = 3 / 100 * size;

    // Colored background
    paint.color = backgroundColor;
    paint.style = PaintingStyle.fill;
    canvas.drawRect(Rect.fromPoints(const Offset(0, 0), Offset(size, size)), paint);

    // centered text value
    final textSpan = TextSpan(
      text: text,
      style: TextStyle(
        color: Colors.black,
        fontSize: size / 2.6,
        fontWeight: FontWeight.bold,
      ),
    );
    final textPainter = TextPainter(
      text: textSpan,
      textDirection: TextDirection.ltr,
      textAlign: TextAlign.center,
    );
    textPainter.layout();
    textPainter.paint(
      canvas,
      Offset(
        (size - textPainter.width) * 0.5,
        (size - textPainter.height) * 0.5,
      ),
    );
  }

  void paintLevelParameterItem(
    final String value,
    final Canvas canvas,
    final double size,
  ) {
    Color backgroundColor = Colors.grey;
    String text = '';

    switch (value) {
      case DefaultGameSettings.levelValueEasy:
        backgroundColor = Colors.green;
        text = '🧒';
        break;
      case DefaultGameSettings.levelValueNormal:
        backgroundColor = Colors.orange;
        text = '🤯';
        break;
      default:
        printlog('Wrong value for level parameter value: $value');
        return;
    }

    final paint = Paint();
    paint.strokeJoin = StrokeJoin.round;
    paint.strokeWidth = 3;

    // Colored background
    paint.color = backgroundColor;
    paint.style = PaintingStyle.fill;
    canvas.drawRect(Rect.fromPoints(const Offset(0, 0), Offset(size, size)), paint);

    // centered text value
    final textSpan = TextSpan(
      text: text,
      style: TextStyle(
        color: Colors.black,
        fontSize: size / 2.6,
        fontWeight: FontWeight.bold,
      ),
    );
    final textPainter = TextPainter(
      text: textSpan,
      textDirection: TextDirection.ltr,
      textAlign: TextAlign.center,
    );
    textPainter.layout();
    textPainter.paint(
      canvas,
      Offset(
        (size - textPainter.width) * 0.5,
        (size - textPainter.height) * 0.5,
      ),
    );
  }
}
